/*
* wmacpimon v0.2.1
* (C) 2002 Costantino Pistagna (valvoline@vrlteam.org)
* wmacpimon homepage: http://www.vlrteam.org/wmacpimon
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <dirent.h>

#include "wmacpimon.h"
#include "acpiintf.h"
#include "parsers.h"

#define MAX(a,b)  ((a)>(b)?(a):(b))
#define MIN(a,b)  ((a)<(b)?(a):(b))

#ifdef ACPI
extern ACPInfo *acpinfo;

/* check if there's something at fName */
static int exists(char *fName)
{
	struct stat fdata;

	return stat(fName, &fdata)==0;
}

void dump_acpinfo(ACPInfo *acpinfo) {
	BattInfo *batt=acpinfo->batteries;

	printf("Temperature (psv/crit): %d (%d/%d)\n",
		acpinfo->temperature, acpinfo->critical_temp,
		acpinfo->passive_temp);
	printf("CPU Speed: %d\n", acpinfo->speed);
	while (batt) {
		printf("Battery %s:\n", batt->statepath);
		printf("  Charge %d of %d, rate %d\n", batt->currcap, batt->fullcap,
			batt->currrate);
		printf("  Charging state: %d\n", batt->charge);
		batt = batt->next;
	}
	printf("Cumulative charge state: %d\n", acpinfo->cum_charge);
	printf("Cumulative percent: %d\n", acpinfo->battery_percent);
	printf("Cumulative remain/full: %d/%d\n", acpinfo->cum_remaining,
		acpinfo->cum_fullcap);
	printf("Remaining %d hrs, %d mins\n", acpinfo->remaining_hours,
		acpinfo->remaining_minutes);
}


/* see if we have ACPI support and check version */
int power_init(void) {
    FILE *acpi;
    char buf[4096];
    int acpi_ver = 0;
    if (!(acpi = fopen("/proc/acpi/info", "r"))) {
		fprintf(stderr, "This system does not support ACPI\n");
		return 1;
    }

    /* okay, now see if we got the right version */
    fread(buf, 4096, 1, acpi);
    acpi_ver = strtol(buf + 25, NULL, 10);
    fprintf(stderr, "ACPI version detected: %d\n", acpi_ver);
    if (acpi_ver < 20020214) {
		fprintf(stderr, "This version requires ACPI subsystem version 20020214 or later\n");
		fclose(acpi);
		return 1;
    }
    /* yep, all good */
    fclose(acpi);

    return 0;
}

	
BattInfo *init_battery(char *battDir) {
	char infopath[PATHLEN];
	char statepath[PATHLEN];
	BattInfo *batt_info=malloc(sizeof(BattInfo));
		/* just crash on failed malloc... */

	snprintf(infopath, PATHLEN, "%s/info", battDir);
	parse_battinfo(infopath, batt_info);
	snprintf(statepath, PATHLEN, "%s/state", battDir);
	batt_info->statepath = strdup(statepath);
	parse_battstate(statepath, batt_info);
	return batt_info;
}


void init_acpinfo(void) {
	BattInfo **batt_chaining;
	int batt_index;

	acpinfo->speed = -1;
	acpinfo->cooling=0;
	acpinfo->state=0;
	acpinfo->batt=0;
	acpinfo->load=0;
	acpinfo->temperature = -1;
	acpinfo->critical_temp = -1;
	acpinfo->passive_temp = -1;
	acpinfo->charge_hist = history_new(10);
		/* crash on failed malloc */
   	parse_thrmtrip("/proc/acpi/thermal_zone/THRM/trip_points", acpinfo);
	batt_chaining = &(acpinfo->batteries);
	*batt_chaining = NULL;
	for (batt_index=0; batt_index<MAX_BATTERIES; batt_index++) {
		char batt_dir[PATHLEN];
		snprintf(batt_dir, PATHLEN, 
			"/proc/acpi/battery/BAT%d", batt_index);
		if (exists(batt_dir)) {
			*batt_chaining = init_battery(batt_dir);
			batt_chaining = &((*batt_chaining)->next);
			*batt_chaining = NULL;
		}
	}

}

void acquire_info() {
    FILE *fd;
	char *buf, *t2;
	
	buf=(char *)malloc(sizeof(char)*512);
	t2=(char *)malloc(sizeof(char)*10);
   	parse_thrmstate("/proc/acpi/thermal_zone/THRM/state", acpinfo);
   	parse_thrmtemp("/proc/acpi/thermal_zone/THRM/temperature", acpinfo);
	fd = fopen("/sys/devices/system/cpu/cpu0/cpufreq/scaling_cur_freq", "r");
	if (fd) {
		fscanf(fd, "%d", &(acpinfo->speed));
		fclose(fd);
		acpinfo->speed /= 1000;
	}
   	parse_thrmcooling("/proc/acpi/thermal_zone/THRM/cooling_mode", acpinfo);
	parse_acstate("/proc/acpi/ac_adapter/AC0/state", acpinfo);
	acpinfo->load=0;
	/*dump_acpinfo(acpinfo);*/
}

void acquire_battinfo() {
	BattInfo *batt_info = acpinfo->batteries;

	while (batt_info) {
		parse_battstate(batt_info->statepath, batt_info);
		batt_info = batt_info->next;
	}
}

void guesstimate_remaining_time() {
	int remaining_seconds = -1;
	double rate;
	
	rate = history_diffaverage(acpinfo->charge_hist);
	acpinfo->current_rate = rate;
	if ((acpinfo->cum_charge==2 && acpinfo->batt==1) || fabs(rate)<1e-7 ||
			isnan(rate)) {
		remaining_seconds = 0;
	} else if (acpinfo->cum_charge==0) {  /* charging */
		remaining_seconds = (int)((acpinfo->cum_fullcap-acpinfo->cum_remaining
				)/rate);
	} else if (acpinfo->cum_charge!=-1) {
		remaining_seconds = (int)(-acpinfo->cum_remaining/rate);
	}
	if (remaining_seconds!=-1) {
		acpinfo->remaining_minutes = (remaining_seconds/60)%60;
		acpinfo->remaining_hours = (remaining_seconds/60)/60;
	} else {
		acpinfo->remaining_minutes = acpinfo->remaining_hours = -1;
	}		
}

void acquire_info2() {
	BattInfo *batt_info = acpinfo->batteries;
	int new_charge = 3;
	acquire_battinfo();

	acpinfo->cum_rate = 0;
	acpinfo->cum_remaining = 0;
	acpinfo->cum_fullcap = 0;
	while (batt_info) {
		acpinfo->cum_rate += batt_info->currrate;
		acpinfo->cum_remaining += batt_info->currcap;
		acpinfo->cum_fullcap += batt_info->fullcap;
		new_charge = MIN(new_charge, batt_info->charge);
		batt_info = batt_info->next;
	}
	history_addval(acpinfo->charge_hist, acpinfo->cum_remaining);
	if (new_charge!=acpinfo->cum_charge) {
		/* History becomes meaningless if charge state changes */
		history_clear(acpinfo->charge_hist);
	}
	acpinfo->cum_charge = new_charge;
	acpinfo->battery_percent = (acpinfo->cum_remaining*100
		)/acpinfo->cum_fullcap;
	guesstimate_remaining_time();
}

#endif /* ACPI */

/* vi:sw=4:ts=4: 
*/
