/*
* wmacpimon v0.2.1
* (C) 2002 Costantino Pistagna (valvoline@vrlteam.org)
* wmacpimon homepage: http://www.vlrteam.org/wmacpimon
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
#define _GNU_SOURCE

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <getopt.h>
#include <unistd.h>
#include <time.h>
#include <math.h>
#include "wmacpimon.h"
#include "acpiintf.h"
#include "master.xpm"
#include "wmgeneral/wmgeneral.c"
#include "wmgeneral/misc.h"

/* let's make life simple...and better! :) */
#define copy_xpm_area(x, y, w, h, dx, dy)				\
{									\
    XCopyArea(dockapp->display, dockapp->pixmap, dockapp->pixmap,	\
	    dockapp->gc, x, y, w, h, dx, dy);				\
    dockapp->update = 1;						\
}

Dockapp *dockapp;
ACPInfo *acpinfo;
int count = 0;			/* global timer variable */
int noisy_critical = 0;	/* ring xbell annoyingly if critical? */


static void redraw_window(void) {
    if (dockapp->update) {
		XCopyArea(dockapp->display, dockapp->pixmap, dockapp->iconwin, 
				dockapp->gc, 0, 0, 64, 64, 0, 0);
		XCopyArea(dockapp->display, dockapp->pixmap, dockapp->win, 
				dockapp->gc, 0, 0, 64, 64, 0, 0);
		dockapp->update = 0;
    }
}

static void new_window(char *name) {
    XpmAttributes attr;
    Pixel fg, bg;
    XGCValues gcval;
    XSizeHints sizehints;
    XClassHint classhint;
    XWMHints wmhints;

    dockapp->screen = DefaultScreen(dockapp->display);
    dockapp->root = DefaultRootWindow(dockapp->display);

    sizehints.flags = USSize;
    sizehints.width = 64;
    sizehints.height = 64;

    fg = BlackPixel(dockapp->display, dockapp->screen);
    bg = WhitePixel(dockapp->display, dockapp->screen);

    dockapp->win = XCreateSimpleWindow(dockapp->display, dockapp->root,
				       0, 0, sizehints.width,
				       sizehints.height, 1, fg, bg);
    dockapp->iconwin =
	XCreateSimpleWindow(dockapp->display, dockapp->win, 0, 0,
			    sizehints.width, sizehints.height, 1, fg, bg);

    XSetWMNormalHints(dockapp->display, dockapp->win, &sizehints);
    classhint.res_name = name;
    classhint.res_class = name;
    XSetClassHint(dockapp->display, dockapp->win, &classhint);

    XSelectInput(dockapp->display, dockapp->win,
		 ExposureMask | ButtonPressMask | ButtonReleaseMask |
		 StructureNotifyMask);
    XSelectInput(dockapp->display, dockapp->iconwin,
		 ExposureMask | ButtonPressMask | ButtonReleaseMask |
		 StructureNotifyMask);

    XStoreName(dockapp->display, dockapp->win, name);
    XSetIconName(dockapp->display, dockapp->win, name);

    gcval.foreground = fg;
    gcval.background = bg;
    gcval.graphics_exposures = False;

    dockapp->gc =
	XCreateGC(dockapp->display, dockapp->win,
		  GCForeground | GCBackground | GCGraphicsExposures,
		  &gcval);

    attr.exactColors = 0;
    attr.alloc_close_colors = 1;
    attr.closeness = 1L << 15;
    attr.valuemask = XpmExactColors | XpmAllocCloseColors | XpmCloseness;
    if (XpmCreatePixmapFromData(dockapp->display, dockapp->win,
				master_xpm, &dockapp->pixmap,
				&dockapp->mask, &attr) != XpmSuccess) {
	fprintf(stderr, "FATAL: Not enough colors for main pixmap!\n");
	exit(1);
    }

    XShapeCombineMask(dockapp->display, dockapp->win, ShapeBounding, 0, 0,
		      dockapp->mask, ShapeSet);
    XShapeCombineMask(dockapp->display, dockapp->iconwin, ShapeBounding, 0,
		      0, dockapp->mask, ShapeSet);

    wmhints.initial_state = WithdrawnState;
    wmhints.flags = StateHint;
    wmhints.icon_window = dockapp->iconwin;
    wmhints.icon_x = sizehints.x;
    wmhints.icon_y = sizehints.y;
    wmhints.window_group = dockapp->win;
    wmhints.flags =
	StateHint | IconWindowHint | IconPositionHint | WindowGroupHint;
    XSetWMHints(dockapp->display, dockapp->win, &wmhints);

    XMapWindow(dockapp->display, dockapp->win);
}

static int open_display(char *display) {
    dockapp->display = XOpenDisplay(display);
    if (!dockapp->display) {
		fprintf(stderr, "Unable to open display '%s'\n", display);
		return 1;
    }
    return 0;
}

static void display_temp(int temp) {
	copy_xpm_area(95, 48, 8, 7, 4, 48);
    if (temp == -1) {
		return;
	}
    if (temp < 0) {
		temp = 0;
	}
    if (temp >= 200) {
		temp = 199;
	}
    if (temp > 100) { /* leading 1 */
		copy_xpm_area(95, 37, 21, 9, 4, 48);
		temp -= 100;
	}
	if (temp >= 10) {
	    copy_xpm_area((temp / 10) * 6 + 67, 28, 5, 7, 7, 49);
	}
	copy_xpm_area((temp % 10) * 6 + 67, 28, 5, 7, 13, 49);
}


static void display_speed(int speed) {
    int i, j, target;
	char display_string[5];

	copy_xpm_area(72, 104, 24, 9, 4, 16);
	if (speed==-1) {
		return;
	}
	snprintf(display_string, 5, "%4d", speed);
	for(i=0, target=5; i<4; i++, target+=6) {
			j = display_string[i];
			if (isdigit(j)) {
				j-=48;
				copy_xpm_area(66+j * 6 + 1, 27, 6, 9, target, 16);
			}
	}
}

static void display_rate(double rate) {
    int i, j, target;
	char display_string[4];

	copy_xpm_area(101, 136, 24, 12, 34, 48);
	if (isnan(rate) || fabs(rate)<1) {
		return;
	}
	if (rate>0) {
		copy_xpm_area(146, 57, 5, 9, 35, 49);
	} else {
		copy_xpm_area(127, 57, 5, 9, 35, 49);
	}
	/* rate is in mWh/sec, which is 1e-3*3.6e3 W */
	snprintf(display_string, 4, "%3d", (int)(rate*3.6));
	for(i=0, target=40; i<3; i++, target+=6) {
			j = display_string[i];
			if (isdigit(j)) {
				j-=48;
				copy_xpm_area(66+j * 6 + 1, 27, 6, 9, target, 49);
			}
	}
}

static void display_charge(int i) {
	if (i == 0) copy_xpm_area(84, 48, 3, 3, 55, 41)
	if (i == 1) copy_xpm_area(90, 48, 3, 3, 55, 41)
	if (i == 2) copy_xpm_area(90, 52, 3, 3, 55, 41)
}

static void display_batt(void) {
	if (acpinfo->batt == 1) {
		copy_xpm_area(117, 37, 12, 7, 43, 36)
	}
	else {
		copy_xpm_area(130, 37, 11, 7, 43, 36)
	}
}


static void display_remainingtime() {
	char dispstr[3];
	int i;

	/* first clear old display, copying form pristine battery area */
	copy_xpm_area(71, 117, 37, 9, 4, 29);
	if (acpinfo->remaining_hours==-1 || (
			acpinfo->remaining_hours==0 && acpinfo->remaining_minutes==0)) {
		return;
	}
	/* copy ":" */
	copy_xpm_area(70, 75, 3, 12, 18, 30);
	/* now pull numbers from the small numbers area */
	snprintf(dispstr, 3, "%2d", acpinfo->remaining_hours);
	for (i=0; i<2; i++) {
		if (isdigit(dispstr[i])) {
			copy_xpm_area(7*(dispstr[i]-'0'), 75, 7, 12, 4+i*8, 30);
		}
	}
	snprintf(dispstr, 3, "%02d", acpinfo->remaining_minutes);
	for (i=0; i<2; i++) {
		if (isdigit(dispstr[i])) {
			copy_xpm_area(7*(dispstr[i]-'0'), 75, 7, 12, 22+i*7, 30);
		}
	}
}


static void display_perc(int percent) {
    static int op = -1, obar;
    unsigned int bar;

    if (op == percent)
	return;

    if (percent < 0)
	percent = 0;
    if (percent > 100)
	percent = 100;

    if (percent < 100) {	/* 0 - 99 */
		copy_xpm_area(95, 48, 8, 7, 36, 17);
		if (percent >= 10) {
	    	copy_xpm_area((percent / 10) * 6 + 67, 28, 5, 7, 39, 17);
		}
		copy_xpm_area((percent % 10) * 6 + 67, 28, 5, 7, 45, 17);
   	} else {
		copy_xpm_area(95, 37, 15, 9, 36, 16);	/* 100% */
	}
    op = percent;
    bar = percent / 1.8518;
    if (bar == obar) {
		return;
	}
    copy_xpm_area(121, 0, bar, 8, 5, 5);
    if (bar < 54) {
		copy_xpm_area(121 + bar, 18, 54 - bar, 8, bar + 5, 4);
	}
    obar = bar;
}


/* compute range such that last 10% are between passive and critical */
int get_temp_percent(ACPInfo *acpinfo)
{
	int fullrange = (acpinfo->critical_temp-acpinfo->passive_temp)*10;
	int zerotemp = acpinfo->critical_temp-fullrange;
	return ((acpinfo->temperature-zerotemp)*100)/fullrange;
}

int main(int argc, char **argv) {
    char *display = NULL;
    char ch;
    int update = 0, update2 = 0;
	int max;
	
    dockapp = calloc(1, sizeof(Dockapp));
    acpinfo = calloc(1, sizeof(ACPInfo));
    dockapp->blink = OFF;
    init_acpinfo();
	
    /* parse command-line options */
    while ((ch = getopt(argc, argv, "d:h")) != EOF) {
		switch (ch) {
			case 'd':
	    		if (optarg) display = strdup(optarg);
	    	break;
			case 'h':
	    		printf("wmacpimon v%s (c) 2002 Costantino Pistagna (valvoline@vrlteam.org)\n"
					"usage: wmacpimon [-d] [-b] [-p] [-c] [-s]\n\n"
		   			"-d display\tdisplay on remote display <display>\n"
		   			"-h\t\tdisplay this help\n\n", ver);
	    	return 0;
	    	break;
		}
	
    }

    /* see if whatever we want to use is supported */
    if (power_init()) exit(1);

    /* open local or command-line specified display */
    if (open_display(display)) exit(1);

    /* make new dockapp window */
    new_window("wmacpimon");

	/* add mouse region */
	AddMouseRegion(1, 0, 0, 64, 64);

 	/* get/set initial statistics */
    acquire_info();
    acquire_info2();
	display_speed(acpinfo->speed);
	display_batt();
	display_charge(acpinfo->cum_charge);
	max = acpinfo->load;

    /* main loop */
    while (1) {
		XEvent event;
		while (XPending(dockapp->display)) {
	    	XNextEvent(dockapp->display, &event);
	    	switch (event.type) {
	    		case Expose: /* update */
					dockapp->update = 1;
					while (XCheckTypedEvent(dockapp->display, Expose, &event));
					redraw_window();
				break;
	    		case DestroyNotify:
					XCloseDisplay(dockapp->display);
					exit(0);
				break;
				case ButtonPress:
					/* CheckMouseRegion(event.xbutton.x, event.xbutton.y);*/
				break;
	    	}
		}
		if (update++ == 20) {
			acquire_info();
			update = 0;
			dockapp->update = 1;
		}		
		if (update2++ == 90) {
	   		acquire_info2();
			update2 = 0;
			dockapp->update = 1;
		}
		display_speed(acpinfo->speed);
		display_batt();
		display_rate(acpinfo->current_rate);
		display_charge(acpinfo->cum_charge);
		display_perc(acpinfo->battery_percent);
		display_temp(acpinfo->temperature);
		display_remainingtime();
		redraw_window();
		usleep(100000);
    }
    return 0;
}

/* vi:sw=4:ts=4: 
*/
